﻿using System;
using Polly;
using Polly.Retry;

class Program
{
    static void Main(string[] args)
    {
        Program p = new Program();
        p.ResultBasedPolicy();
    }

    private void ResultBasedPolicy()
    {
        ErrorProneCode errorProneCode = new ErrorProneCode();

        ISyncPolicy<Status> waitAndRetryPolicyHandlesResult = Policy.HandleResult<Status>(s => s != Status.Success)
            .WaitAndRetry(5,
            (retryCount, status, ctx) =>
            {
                if (status.Result == Status.Unknown)
                {
                    return TimeSpan.FromSeconds(1);
                }
                else
                {
                    return TimeSpan.FromSeconds(2);
                }
            },
            onRetry: (status, timeSpan, retryCount, ctx) =>
                {
                    Console.WriteLine($"Got a response of {status.Result}, retrying {retryCount}. Delaying for {timeSpan}");
                }
            );

        Status result = waitAndRetryPolicyHandlesResult.Execute(errorProneCode.GetStatus);

        Console.WriteLine(result);
    }

}